////////////////////////////////////////////////////////////////////// 
// merlin/MerlinSimwalk2.cpp 
// (c) 2000-2001 Goncalo Abecasis
// 
// This file is distributed as part of the MERLIN source code package   
// and may not be redistributed in any form, without prior written    
// permission from the author. Permission is granted for you to       
// modify this file for your own personal use, but modified versions  
// must retain this copyright notice and must not be distributed.     
// 
// Permission is granted for you to use this file to compile MERLIN.    
// 
// All computer programs have bugs. Use this file at your own risk.   
// 
// Monday December 17, 2001
// 
 
#include "MerlinFamily.h"
#include "MerlinSimwalk2.h"

#include <math.h>

void NPLforSimwalk2::OpenFiles(const char * name)
   {
   String filename(name);
   filename += ".npl";
   output = fopen(filename, "wt");
   }

void NPLforSimwalk2::CloseFiles()
   {
   fclose(output);
   printf("Intermediate output for Simwalk2 stored in file [merlin.npl]\n");
   }

void NPLforSimwalk2::Output()
   {
   if ((family.nplInv[0] == 0) && (family.nplInv[1] == 0))
      {
      Skip();
      return;
      }

   // Header line has pedigree name (cols 1-8),
   // and no. of markers (cols. 11 - 15)
   fprintf(output, "%8.8s%7d\n",
          (const char *) family.family->famid, family.markerCount);

   // Headings for the table to follow
   fprintf(output, "%-8s  %9s %9s %9s %9s %9s %9s\n",
          "Marker",
          "Obs_Pair", "Mean_Pair", "SDev_Pair",
          "Obs_All", "Mean_All", "SDev_All");

   // Simwalk2 expects the statistics to be in a slightly different scale
   int count = family.mantra.aff_count;

   // Standardization for pairs statistic is SQRT(NAFF*(NAFF-1)/2)/4
   double pairsScale = sqrt(count * (count - 1) * 0.125);

   // Standardization for all statistic is SQRT(NAFF!)
   double allScale = 1.0;
   for (int n = 1; n < count; n++)
      allScale *= n;
   allScale = sqrt(allScale);

   // Mean and variance for each statistic
   double pairsMean = family.nplMean[1] * pairsScale;
   double pairsDev = family.nplInv[1] ? 1.0/family.nplInv[1] * pairsScale : 0.0;

   double allMean = family.nplMean[0] * allScale;
   double allDev = family.nplInv[0] ? 1.0/family.nplInv[0] * allScale : 0.0;

   // When there are multiple markers at the same location, MERLIN
   // only calculates one score, but SIMWALK2 expects one score per
   // marker.
   //
   // We iterate over all markers i and output scores for the same
   // analysis position multiple times if required
   //
   int pos = 0;

   for (int i = 0; i < family.markers.Length(); i++)
      {
      // Find position corresponding to current marker
      while (family.analysisPositions[pos] < family.markerPositions[i])
         pos++;

      // Output scores
      fprintf(output, "%-8s ", (const char *) family.ped.markerNames[family.markers[i]]);
      OutputValue(family.kac.scores[1][family.serial][pos] * pairsDev + pairsMean);
      OutputValue(pairsMean);
      OutputValue(pairsDev);
      OutputValue(family.kac.scores[0][family.serial][pos] * allDev + allMean);
      OutputValue(allMean);
      OutputValue(allDev);
      fprintf(output, "\n");
      }
   }

void NPLforSimwalk2::Skip()
   {
   // Header line has pedigree name (cols 1-8) and zero to denote skipping
   fprintf(output, "%8.8s%7d\n", (const char *) family.family->famid, 0);
   }


void NPLforSimwalk2::OutputValue(double value)
   {
   if (value < 1e-10)
      fprintf(output, "         0");
   else
      fprintf(output, " %9.*G", (value > 1000000 || value < 1.0) ? 4 : 7, value);
   }

 
